class VirtualenvFeature(object):
    def __init__(self, dispatcher):
        dispatcher.add_listener('edgy.project.on_start', self.on_start, priority=-99)

    def on_start(self, event):
        pass # todo ? or makefile responsability ? or bundled in python feature ?

__feature__ = VirtualenvFeature



from __future__ import absolute_import

from . import Feature


class Flake8Feature(Feature):
    def configure(self):
        self.dispatcher.add_listener('edgy.project.feature.make.on_generate', self.on_make_generate)

    def on_make_generate(self, event):
        makefile = event.makefile
        makefile.add_target('lint', '''
            $(VIRTUALENV_PATH)/bin/flake8 {name}
        '''.format(name=event.package_name), deps=('install', ), phony=True)


__feature__ = Flake8Feature



from __future__ import absolute_import

import os

from jinja2 import Environment, PackageLoader
from jinja2 import Template

from edgy.project.file import File
from edgy.project.util import format_file_content


class Feature(object):
    _jinja_environment = None

    def __init__(self, dispatcher):
        self.dispatcher = dispatcher
        self.configure()

    def configure(self):
        pass

    def file(self, *args, **kwargs):
        return File(self.dispatcher, *args, **kwargs)

    @property
    def jinja(self):
        if type(self)._jinja_environment is None:
            type(self)._jinja_environemnt = Environment(
                loader=PackageLoader(__name__, 'template')
            )
        return type(self)._jinja_environemnt

    def render(self, template, context=None):
        context = context or {}
        os.path.join(os.path.dirname(__file__), 'template')
        return self.jinja.get_template(template).render(**(context or {}))

    def render_file(self, target, template, context=None, override=False):
        with self.file(target, override=override) as f:
            f.write(format_file_content(self.render(template, context)))

    def render_file_inline(self, target, template_string, context=None, override=False):
        with self.file(target, override=override) as f:
            f.write(format_file_content(Template(template_string).render(**(context or {}))))

    def render_empty_files(self, *targets, **kwargs):
        for target in targets:
            with self.file(target, override=kwargs.pop('override', False)) as f:
                pass



from __future__ import absolute_import

import os
from . import Feature

class GitFeature(Feature):
    def configure(self):
        self.dispatcher.add_listener('edgy.project.on_start', self.on_start, priority=-100)
        self.dispatcher.add_listener('edgy.project.on_file_closed', self.on_file_change)
        self.dispatcher.add_listener('edgy.project.on_end', self.on_end)

    def on_start(self, event):
        if not os.path.exists('.git'):
            os.system('git init')
            os.system('git add Projectfile')
            os.system('git commit -m "initial commit"')

    def on_end(self, event):
        self.render_file_inline('.gitignore', '''
            *.egg-info
            *.pyc
            *.swp
            /.cache
            /.coverage
            /.idea
            /.python*-*
            /build
            /dist
            /htmlcov
            /pylint.html
        ''', event.variables)

    def on_file_change(self, event):
        os.system('git add {}'.format(event.filename))


__feature__ = GitFeature



from __future__ import absolute_import

from . import Feature


class NosetestsFeature(Feature):
    def configure(self):
        self.dispatcher.add_listener('edgy.project.feature.make.on_generate', self.on_make_generate)
        self.dispatcher.add_listener('edgy.project.feature.python.on_generate', self.on_python_generate)

    def on_make_generate(self, event):
        makefile = event.makefile
        makefile.add_target('test', '''
            $(VIRTUALENV_PATH)/bin/nosetests -q --with-doctest --with-coverage --cover-package={name}
        '''.format(name=event.package_name), deps=('install', ), phony=True)

    def on_python_generate(self, event):
        event.files['requirements'] = '\n'.join((event.files['requirements'], 'nose >=1.3,<1.4', 'coverage >=3.7,<3.8' ))


__feature__ = NosetestsFeature



#!/usr/bin/env python
# coding=utf-8

from __future__ import unicode_literals, absolute_import, print_function

import os

from collections import OrderedDict
from edgy.event import Event, EventDispatcher

from .config import read_configuration

class ArgumentParser(object):
    from argparse import ArgumentParser as ArgumentParserType

    def __init__(self):
        self.parser = type(self).ArgumentParserType()
        self.subparsers = self.parser.add_subparsers(dest="command")
        init_parser = self.subparsers.add_parser('init')

    def parse_args(self, args=None, namespace=None):
        return self.parser.parse_args(args, namespace)

def _read_configuration(dispatcher):
    config_filename = os.path.join(os.getcwd(), 'Projectfile')
    if not os.path.exists(config_filename):
        raise IOError('Could not find project description file (looked in {})'.format(config_filename))

    variables = OrderedDict((
        ('virtualenv_path', '.virtualenv-$(PYTHON_BASENAME)', ),
    ))

    files = {
        'requirements': '',
        'classifiers': '',
        'version': '',
    }

    setup = OrderedDict((
        ('name', None, ),
        ('description', None, ),
        ('license', None, ),
        ('url', 'http://example.com/', ),
        ('download_url', 'http://example.com/', ),
        ('extras_require', {}, ),
    ))

    features = {
        'git',
        'make',
        'pytest',
        'python',
        'pylint',
    }

    return read_configuration(dispatcher, config_filename, variables, features, files, setup)

class ProjectEvent(Event):
    variables = None
    files = None
    setup = None

def main(args=None):
    parser = ArgumentParser()
    dispatcher = EventDispatcher()

    variables, features, files, setup = _read_configuration(dispatcher)
    feature_instances = {}
    for feature_name in sorted(features):
        try:
            feature = __import__('edgy.project.feature.' + feature_name, fromlist=('__feature__', )).__feature__
        except (ImportError, AttributeError, ) as e:
            print('Error while importing feature "{}" ...'.format(feature_name))
            raise

        if feature:
            feature_instances[feature_name] = feature(dispatcher)

    event = ProjectEvent()
    event.variables, event.files, event.setup = variables, files, setup
    event = dispatcher.dispatch('edgy.project.on_start', event)
    event = dispatcher.dispatch('edgy.project.on_end', event)


if __name__ == '__main__':
    main()



from __future__ import absolute_import, unicode_literals, print_function

import datetime
import six
import textwrap

from collections import deque

from edgy.event import Event
from edgy.project.feature import Feature

if six.PY3:
    unicode = str

class Makefile(object):
    def __init__(self):
        self._env_order, self._env_values = deque(), {}
        self._target_order, self._target_values = deque(), {}
        self.phony = set()

    def add_target(self, target, rule, deps=None, phony=False, first=False, doc=None):
        self._target_values[target] = (
            deps or list(),
            textwrap.dedent(rule).strip(),
            textwrap.dedent(doc or '').strip(),
        )
        self._target_order.appendleft(target) if first else self._target_order.append(target)

        if phony:
            self.phony.add(target)

    def set_deps(self, target, deps=None):
        self._target_values[target] = (
            deps or list(),
            self._target_values[target][1],
            self._target_values[target][2],
        )

    def __str__(self):
        content = [
            '# This file has been auto-generated.',
            '# All manual changes may be lost, see Projectfile.',
            '#',
            '# Date: '+ unicode(datetime.datetime.now()),
            '',
        ]

        if len(self):
            for k, v in self:
                content.append('{} ?= {}'.format(k, v))
            content.append('')

        if len(self.phony):
            content.append('.PHONY: ' + ' '.join(sorted(self.phony)))
            content.append('')

        for target, details in self.targets:
            deps, rule, doc = details
            if doc:
                for line in doc.split('\n'):
                    content.append('# ' + line)
            content.append('{}: {}'.format(target, ' '.join(deps)).strip())
            for line in rule.split('\n'):
                content.append('	' + line)
            content.append('')

        return '\n'.join(content)

    def __setitem__(self, key, value):
        self._env_values[key] = value
        if not key in self._env_order:
            self._env_order.append(key)

    def setleft(self, key, value):
        self._env_values[key] = value
        if not key in self._env_order:
            self._env_order.appendleft(key)

    def updateleft(self, *lst):
        for key, value in reversed(lst):
            self.setleft(key, value)

    def __getitem__(self, item):
        return self._env_values[item]

    def __delitem__(self, key):
        self._env_order.remove(key)
        del self._env_values[key]

    def __len__(self):
        return len(self._env_order)

    def __iter__(self):
        for key in self._env_order:
            yield key, self._env_values[key]

    @property
    def targets(self):
        for key in self._target_order:
            yield key, self._target_values[key]


class MakefileEvent(Event):
    def __init__(self, package_name, makefile):
        self.package_name = package_name
        self.makefile = makefile
        super(MakefileEvent, self).__init__()


class MakeFeature(Feature):
    def configure(self):
        self.makefile = Makefile()
        self.dispatcher.add_listener('edgy.project.on_start', self.on_start)

    def on_start(self, event):
        for k in event.variables:
            self.makefile[k.upper()] = event.variables[k]

        self.makefile.updateleft(
            ('PYTHON', '$(shell which python)', ),
            ('PYTHON_BASENAME', '$(shell basename $(PYTHON))', ),
            ('PYTHON_REQUIREMENTS_FILE', 'requirements.txt', ),
            ('QUICK', '', ),
        )

        self.makefile['PIP'] = '$(VIRTUALENV_PATH)/bin/pip'

        # Install
        self.makefile.add_target('install', '''
            if [ -z "$(QUICK)" ]; then \\
                $(PIP) install -Ue "file://`pwd`#egg={name}[dev]"; \\
            fi
        '''.format(name=event.setup['name']), deps=('$(VIRTUALENV_PATH)', ),
                                 phony=True, doc='''
            Installs the local project dependencies.
        ''')

        # Virtualenv
        self.makefile.add_target('$(VIRTUALENV_PATH)', '''
            virtualenv -p $(PYTHON) $(VIRTUALENV_PATH)
            $(PIP) install -U pip\>=8.0,\<9.0 wheel\>=0.24,\<1.0
            ln -fs $(VIRTUALENV_PATH)/bin/activate activate-$(PYTHON_BASENAME)
        ''', doc='''
            Setup the local virtualenv.
        ''')

        # Housekeeping
        self.makefile.add_target('clean', '''
            rm -rf $(VIRTUALENV_PATH)
        ''', phony=True)

        self.dispatcher.dispatch(__name__ + '.on_generate', MakefileEvent(event.setup['name'], self.makefile))

        self.render_file_inline('Makefile', self.makefile.__str__(), override=True)


__feature__ = MakeFeature



from __future__ import absolute_import

from . import Feature


class PylintFeature(Feature):
    def configure(self):
        self.dispatcher.add_listener('edgy.project.feature.make.on_generate',
                                     self.on_make_generate, priority=-50)
        self.dispatcher.add_listener('edgy.project.feature.python.on_generate',
                                     self.on_python_generate, priority=-50)

    def on_make_generate(self, event):
        makefile = event.makefile
        makefile.add_target('lint', '''
            $(VIRTUALENV_PATH)/bin/pylint --py3k {name} -f html > pylint.html
        '''.format(name=event.package_name), deps=('install', ), phony=True)

    def on_python_generate(self, event):
        event.files['requirements'] = '\n'.join((event.files['requirements'], 'pylint >=1.4,<1.5', ))


__feature__ = PylintFeature



from __future__ import absolute_import

import os

from . import Feature

class PythonFeature(Feature):
    def configure(self):
        self.dispatcher.add_listener('edgy.project.on_start', self.on_start, priority=-99)

    def on_start(self, event):
        self.dispatcher.dispatch(__name__+'.on_generate', event)

        self.render_empty_files('classifiers.txt', 'requirements.txt', 'version.txt', 'README.rst')
        self.render_file_inline('MANIFEST.in', 'include *.txt')
        self.render_file_inline('setup.cfg', '''
                [metadata]
                description-file = README.rst
            ''')

        # Explode package name so we know which python packages are namespaced and
        # which are not
        package_bits = event.setup['name'].split('.')
        if len(package_bits) > 1:
            event.setup['namespace_packages'] = []
            for i in range(1, len(package_bits)):
                event.setup['namespace_packages'].append('.'.join(package_bits[0:i]))

        for i in range(1, len(package_bits)+1):
            _bits = package_bits[0:i]
            package_dir = os.path.join(*_bits)
            if not os.path.exists(package_dir):
                os.makedirs(package_dir)

            package_init_file = os.path.join(package_dir, '__init__.py')
            if not os.path.exists(package_init_file):
                self.render_file(package_init_file, 'python/package_init.py.j2', {'is_namespace': i < len(package_bits)})

        self.render_file('setup.py', 'python/setup.py.j2', {
            'setup': event.setup,
            'url': event.setup.pop('url', 'http://example.com/'),
            'download_url': event.setup.pop('download_url', 'http://example.com/'),
            'extras_require': event.setup.pop('extras_require', {})
        }, override=True)

__feature__ = PythonFeature
